module alu(c_out,a_in,b_in,f_add,f_sub,f_and,f_or,f_not,f_xor,f_cmp,f_br,f_jmp,f_ld,f_ldi,f_ldx,f_st,f_stx,ir);	
   parameter width = 32, prop = 25;   
   input [width-1:0] a_in;   // The ALU input for operand 1.
   input [width-1:0] b_in;   // The ALU input for operand 2.
   input 	     f_add;  // Instruction identification bit.
   input 	     f_sub;  //      "           "          "
   input 	     f_or;   //      "           "          "
   input 	     f_not;  //      "           "          "
   input 	     f_and;  //      "           "          "
   input 	     f_xor;  //      "           "          "
   input 	     f_cmp;  //      "           "          "
   input 	     f_br;   //      "           "          "
   input 	     f_jmp;  //      "           "          "
   input 	     f_ld;   //      "           "          "
   input 	     f_ldi;  //      "           "          "
   input 	     f_ldx;  //      "           "          "
   input 	     f_st;   //      "           "          "
   input 	     f_stx;  //      "           "          "
   input 	     ir;     // Unused. Why?
   output [width:0]  c_out;  // The ALU result.
   
   reg [width:0]     c_out;      
   
   always
     @(a_in or b_in or f_add or f_sub or f_and or f_or or f_not or f_cmp or
       f_br or f_jmp or f_ld or f_ldi or f_ldx or f_st or f_stx or f_xor)  
     begin
	#1;
	if(f_add)
	  begin
	     #(prop) c_out = a_in + b_in;
	  end
	if(f_sub)
	  begin
	     #(prop) c_out = a_in - b_in;
	  end
	if(f_not)
	  begin
	     #(prop) c_out[width-1:0] = ~a_in;
	     c_out[width] = 0;
	  end
	if(f_and)
	  begin
	     #(prop) c_out = a_in & b_in;
	  end
	if(f_or)
	  begin
	     #(prop) c_out = a_in | b_in;
	  end
	if(f_cmp)
	  begin
	     #(prop) c_out = a_in - b_in;
	  end
	if(f_br)
	  begin
	     #(prop) c_out =  a_in + b_in;
	  end     
	if(f_jmp)
	  begin
	     #(prop) c_out = a_in + b_in;
	  end     
	if(f_ld || f_ldi)
	  begin
	     #(prop) c_out =  b_in;
	  end
	if(f_ldx)
	  begin
	     #(prop) c_out = a_in + b_in;
	  end
	if(f_st)
	  begin
	     #(prop) c_out =  b_in;
	  end
	if(f_stx)
	  begin
	     #(prop) c_out = a_in + b_in;
	  end
	if(f_xor)
	  begin
             #(prop) c_out = a_in ^ b_in;
	  end   
     end   
endmodule // alu




