//
// Test the basic operation of all of the instructions in the ISA.
//
// When the program terminates, register r0 should be contain 0xffffffff
// and the PC should contain 0x1008.
// If the PC contains any other value, r0 should contain an error
// code corresponding to the error code in the comment next to the
// instruction that caused the error.
// 

	.org 0

; Test a few basic instructions first.
;

	ldi r0,#0xf
	cmp r0, #15
	beq start
	hlt		; if you ended up here, not much is working

start:


; Test that every register can be read and written correctly.

	ldi r0,#0
	cmp r0,#0
	bne error_regs
	ldi r1,#1
	cmp r1,#1
	bne error_regs
	ldi r2,#2
	cmp r2,#2
	bne error_regs
	ldi r3,#3
	cmp r3,#3
	bne error_regs
	ldi r4,#4
	cmp r4,#4
	bne error_regs
	ldi r5,#5
	cmp r5,#5
	bne error_regs
	ldi r6,#6
	cmp r6,#6
	bne error_regs
	ldi r7,#7
	cmp r7,#7
	bne error_regs
	ldi r8,#8
	cmp r8,#8
	bne error_regs
	ldi r9,#9
	cmp r9,#9
	bne error_regs
	ldi r10,#10
	cmp r10,#10
	bne error_regs
	ldi r11,#11
	cmp r11,#11
	bne error_regs
	ldi r12,#12
	cmp r12,#12
	bne error_regs
	ldi r13,#13
	cmp r13,#13
	bne error_regs
	ldi r14,#14
	cmp r14,#14
	bne error_regs
	ldi r15,#15
	cmp r15,#15
	bne error_regs
	ldi r16,#16
	cmp r16,#16
	bne error_regs
	ldi r17,#17
	cmp r17,#17
	bne error_regs
	ldi r18,#18
	cmp r18,#18
	bne error_regs
	ldi r19,#19
	cmp r19,#19
	bne error_regs
	ldi r20,#20
	cmp r20,#20
	bne error_regs
	ldi r21,#21
	cmp r21,#21
	bne error_regs
	ldi r22,#22
	cmp r22,#22
	bne error_regs
	ldi r23,#23
	cmp r23,#23
	bne error_regs
	ldi r24,#24
	cmp r24,#24
	bne error_regs
	ldi r25,#25
	cmp r25,#25
	bne error_regs
	ldi r26,#26
	cmp r26,#26
	bne error_regs
	ldi r27,#27
	cmp r27,#27
	bne error_regs
	ldi r28,#28
	cmp r28,#28
	bne error_regs
	ldi r29,#29
	cmp r29,#29
	bne error_regs
	ldi r30,#30
	cmp r30,#30
	bne error_regs
	ldi r31,#31
	cmp r31,#31
	bne error_regs

; Test various combinations of the add instruction.

	; Two positive values
	ldi r0,#9
	ldi r1,#11
	add r2, r0, r1
	cmp r2,#20
	bne error_add

	; Two negative values
	ldi r0,#-46
	ldi r1,#-98
	add r2, r0, r1
	cmp r2,#-144
	bne error_add

	; One positive, one negative value
	ldi r0,#33
	ldi r1,#-12
	add r2, r0, r1
	cmp r2,#21
	bne error_add

	; One negative, one positive value
	ldi r0,#-67
	ldi r1,#98
	add r2, r0, r1
	cmp r2,#31
	bne error_add

	; Test immediate operand form.
	ldi r0,#99
	add r2, r0, #34
	cmp r2,#133
	bne error_add

	ldi r0,#99
	add r2, r0, #-34
	cmp r2,#65
	bne error_add


; Test various combinations of the sub instruction.

	; Two positive values
	ldi r0,#9
	ldi r1,#11
	sub r2, r0, r1
	cmp r2,#-2
	bne error_sub

	; Two negative values
	ldi r0,#-46
	ldi r1,#-98
	sub r2, r0, r1
	cmp r2,#52
	bne error_sub

	; One positive, one negative value
	ldi r0,#33
	ldi r1,#-12
	sub r2, r0, r1
	cmp r2,#45
	bne error_sub

	; One negative, one positive value
	ldi r0,#-67
	ldi r1,#98
	sub r2, r0, r1
	cmp r2,#-165
	bne error_sub

	; Test immediate operand form.
	ldi r0,#99
	sub r2, r0, #34
	cmp r2,#65
	bne error_sub

	ldi r0,#99
	sub r2, r0, #-34
	cmp r2,#133
	bne error_sub




; Test the combinations of the and instruction.

	ldi r2,#0xa5555		; clear out result reg
	ldi r0,#-1
	ldi r1,#-1
	and r2, r0, r1
	cmp r2,#-1
	bne error_and

	ldi r2,#0xa5555		; clear out result reg
	ldi r0,#-1
	ldi r1,#0
	and r2, r0, r1
	cmp r2,#0
	bne error_and

	ldi r2,#0xa5555		; clear out result reg
	ldi r0,#0
	ldi r1,#-1
	and r2, r0, r1
	cmp r2,#0
	bne error_and

	ldi r2,#0xa5555		; clear out result reg
	ldi r0,#0
	ldi r1,#0
	and r2, r0, r1
	cmp r2,#0
	bne error_and

; Test the combinations of the or instruction.

	ldi r2,#0xa5555		; clear out result reg
	ldi r0,#-1
	ldi r1,#-1
	or r2, r0, r1
	cmp r2,#-1
	bne error_or

	ldi r2,#0xa5555		; clear out result reg
	ldi r0,#-1
	ldi r1,#0
	or r2, r0, r1
	cmp r2,#-1
	bne error_or

	ldi r2,#0xa5555		; clear out result reg
	ldi r0,#0
	ldi r1,#-1
	or r2, r0, r1
	cmp r2,#-1
	bne error_or

	ldi r2,#0xa5555		; clear out result reg
	ldi r0,#0
	ldi r1,#0
	or r2, r0, r1
	cmp r2,#0
	bne error_or

; Test the combinations of the xor instruction
	
	ldi r2,#0xa5555		; clear out result reg
	ldi r0,#-1
	ldi r1,#-1
	xor r2, r0, r1
	cmp r2,#0
	bne error_xor

	ldi r2,#0xa5555		; clear out result reg
	ldi r0,#-1
	ldi r1,#0
	xor r2, r0, r1
	cmp r2,#-1
	bne error_xor

	ldi r2,#0xa5555		; clear out result reg
	ldi r0,#0
	ldi r1,#-1
	xor r2, r0, r1
	cmp r2,#-1
	bne error_xor

	ldi r2,#0xa5555		; clear out result reg
	ldi r0,#0
	ldi r1,#0
	xor r2, r0, r1
	cmp r2,#0
	bne error_xor
	 
; Test the combinations of the not instruction.

	ldi r2,#0xa5555		; clear out result reg
	ldi r0,#-1
	not r2, r0
	cmp r2,#0
	bne error_not

	ldi r2,#0xa5555		; clear out result reg
	ldi r0,#0
	not r2, r0
	cmp r2,#-1
	bne error_not


; Test the various conditional branches

	bra b1
b2:	hlt			; bra or bnv failed, give up and die
b1:	bnv b2

; In the following loop tests, r0 counts the number of passes made
; through the loop.  This count will vary depending on the
; specific exit condition.  It used to determine if the loop
; branched the correct number of times for the given branch condition.

LOOPCNT	.equ	5

	; Test bne
	ldi r0,#0		; number of passes through the loop
	ldi r1,#LOOPCNT		; iteration counter
loop1:	add r0,r0,#1		; passes through the loop
	sub r1,r1,#1		; decrement iteration counter
	cmp r1,#0
	bne loop1		; loop until condition met
	cmp r0,#LOOPCNT		; verify passed through loop correct
				; number of times
	bne error_bxx

	; Test bge
	ldi r0,#0		; number of passes through the loop
	ldi r1,#LOOPCNT		; iteration counter
loop2:	add r0,r0,#1		; passes through the loop
	sub r1,r1,#1		; decrement iteration counter
	cmp r1,#0
	bge loop2		; loop until condition met
	cmp r0,#LOOPCNT+1	; verify passed through loop correct
				; number of times
	bne error_bxx

	; Test beq
	ldi r0,#0		; number of passes through the loop
	ldi r1,#LOOPCNT		; iteration counter
loop3:	add r0,r0,#1		; passes through the loop
	sub r1,r1,#1		; decrement iteration counter
	cmp r1,#0
	beq done3		; loop until condition met
	bra loop3
done3:	cmp r0,#LOOPCNT		; verify passed through loop correct
				; number of times
	bne error_bxx

	; Test blt
	ldi r0,#0		; number of passes through the loop
	ldi r1,#0		; iteration counter
loop4:	add r0,r0,#1		; passes through the loop
	add r1,r1,#1		; decrement iteration counter
	cmp r1,#LOOPCNT
	blt loop4		; loop until condition met
	cmp r0,#LOOPCNT		; verify passed through loop correct
				; number of times
	bne error_bxx


	; Test bgt
	ldi r0,#0		; number of passes through the loop
	ldi r1,#LOOPCNT		; iteration counter
loop5:	add r0,r0,#1		; passes through the loop
	sub r1,r1,#1		; decrement iteration counter
	cmp r1,#0
	bgt loop5		; loop until condition met
	cmp r0,#LOOPCNT		; verify passed through loop correct
				; number of times
	bne error_bxx

	; Test ble
	ldi r0,#0		; number of passes through the loop
	ldi r1,#0		; iteration counter
loop6:	add r0,r0,#1		; passes through the loop
	add r1,r1,#1		; decrement iteration counter
	cmp r1,#LOOPCNT
	ble loop6		; loop until condition met
	cmp r0,#LOOPCNT+1	; verify passed through loop correct
				; number of times
	bne error_bxx

	; Test bpl
	ldi r0,#0		; number of passes through the loop
	ldi r1,#LOOPCNT		; iteration counter
loop7:	add r0,r0,#1		; passes through the loop
	sub r1,r1,#1		; decrement iteration counter
	cmp r1,#0
	bpl loop7		; loop until condition met
	cmp r0,#LOOPCNT+1	; verify passed through loop correct
				; number of times
	bne error_bxx

	; Test bmi
	ldi r0,#0		; number of passes through the loop
	ldi r1,#0-LOOPCNT	; iteration counter
loop8:	add r0,r0,#1		; passes through the loop
	add r1,r1,#1		; decrement iteration counter
	cmp r1,#0
	bmi loop8		; loop until condition met
	cmp r0,#LOOPCNT		; verify passed through loop correct
				; number of times
	bne error_bxx

	; Test bvc and bvs
	; Keep doubling value until overflow, count number of iterations.
	ldi r0,#0		; number of passes through the loop
	ldi r1,#1		; iteration counter
loop9:	add r0,r0,#1		; passes through the loop
	add r1,r1,r1		; double value
	bvc loop9		; loop until condition met
	bvs check9
	bra error_bxx
check9:	cmp r0,#31		; verify passed through loop correct
				; number of times
	bne error_bxx
	
	; Test bcc and bcs
	; Keep doubling value until overflow, count number of iterations.
	ldi r0,#0		; number of passes through the loop
	ldi r1,#1		; iteration counter
loop10:	add r0,r0,#1		; passes through the loop
	add r1,r1,r1		; double value
	bcc loop10		; loop until condition met
	bcs check10
	bra error_bxx
check10: cmp r0,#32		; verify passed through loop correct
				; number of times
	bne error_bxx
	

; Test jmp and jmpl

	; Test jmp through a register
	ldi r0,#jmp_tst1	; jump around the branch
	jmp r0
	bra error_jmp

jmp_tst1:
	; test jmp through a register with offset
	ldi r0,#jmp_tst2
	jmp r0,#12

jmp_tst2:
	bra error_jmp
	nop
	nop
	nop
	nop
	nop

	; test jmp and link
	ldi r30,#0		; clear returned value register
	ldi r0,#jmp_tst3
	jmpl r31,r0		; jump and save return address
	cmp r30,#0xf5		; was the correct value returned?
	bne error_jmp		; if not, goto error exit

	; test jmp and link with offsett
	ldi r30,#0		; clear returned value register
	ldi r0,#jmp_tst4
	jmpl r31,r0,#8		; jump and save return address
	cmp r30,#0xf5		; was the correct value returned?
	bne error_jmp		; if not, goto error exit

	; branch around the jmpl tests
	bra around_jmpl_tsts

jmp_tst3:
	ldi r30,#0xf5		; store a value to check on return
	jmp r31			; return to the stored adress

jmp_tst4:
	ldi r30,#0xaa		; store the wrong value to check on return
	jmp r31			; return to the stored adress
	ldi r30,#0xf5		; store the right value to check on return
	jmp r31			; return to the stored adress

around_jmpl_tsts:


; Test the mov pseudo operation.

	ldi r0,#0xdaad
	ldi r1,#0xadda
	cmp r0,r1
	beq error_mov
	mov r0,r1
	cmp r0,r1
	bne error_mov



; Test the load and store instructions.

	; Test ld
	ldi r0,#0		; clear r0
	ld r0,aaa		; move the value from address aaa into r0
	cmp r0,#0x1a		; make sure it has the expected value
	bne error_ldst

	; Test ldx
	ldi r0,#0		; clear r0
	ldi r15,#aaa		; put the address of aaa into r15
	add r15,r15,#8		; point to two words after aaa
	ldx r0,r15		; move the value from this address into r0
	cmp r0,#0x1c		; make sure it has the expected value
	bne error_ldst

	; Now verify that the offset works
	ldx r0,r15,#20		; read mem[r15+20]
	cmp r0,#0x2b		; make sure it has the expected value
	bne error_ldst

	; Test st
	ldi r5,#0xb0b		; a value to store
	st aaa,r5		; overwrite value at addr aaa
	ldi r0,#0		; clear r0
	ld r0,aaa		; move the value from address aaa into r0
	cmp r0,#0xb0b		; make sure it has the expected value
	bne error_ldst

	; Test stx
	ldi r5,#0xdad		; a value to store
	ldi r15,#aaa		; put the address of aaa into r15
	add r15,r15,#12		; point to three words after aaa
	stx r15,r5		; store the value from r5 into mem[r15]
	ldi r0,#0		; clear r0
	ldx r0,r15		; move the value from this address into r0
	cmp r0,#0xdad		; make sure it has the expected value
	bne error_ldst

	; Now verify that the offset works.
	ldi r5,#0xace		; a value to store
	ldi r15,#aaa		; put the address of aaa into r15
	stx r15,#24,r5		; store the value into mem[r15+24]
	ldi r0,#0		; clear r0
	ldx r0,r15,#24		; move the value from this address into r0
	cmp r0,#0xace		; make sure it has the expected value
	bne error_ldst


;
; If you got this far, no errors were found.
;
	
	ldi r0,#good_exit
	jmp r0



; If you stopped here, some sort of error was detected.
;
error_exits:

error_regs:
	ldi r0,#0x2
	hlt

error_add:
	ldi r0,#0x3
	hlt

error_sub:
	ldi r0,#0x4
	hlt

error_and:
	ldi r0,#0x5
	hlt

error_or:
	ldi r0,#0x6
	hlt

error_not:
	ldi r0,#0x7
	hlt

error_bxx:
	ldi r0,#0x8
	hlt

error_jmp:
	ldi r0,#0x9
	hlt

error_mov:
	ldi r0,#0xa
	hlt

error_ldst:
	ldi r0,#0xb
	hlt

error_xor:
	ldi r0, #0xc
	hlt
	
; Here are some constants and other memory locations used for
; testing the load/store instructions.

aaa:	.word	0x1a
	.word	0x1b
	.word	0x1c
	.word	0x1d
	.word	0x1e
	.word	0x1f
	.word	0x2a
	.word	0x2b
	.word	0x2c
	.word	0x2d
	.word	0x2e
	.word	0x2f


; If you stopped here, things worked OK.
;
	.org 0x1000
good_exit:
	ldi r0,#-1
	hlt







