module stage4(IR5, Z5, WR, BR4, JMP4, ADDR_IN2, D_IN2, CLK, clr, IR4, PC4, Z4, MD4, D_OUT2, c11, RW4, HALT4, RW5, HALT5);   
   input 	 CLK;      // The clock.
   input 	 clr;      // The signal to clear all D-type flip-flops.
   input 	 c11;      // Mux selector for source of data write. 
   input [31:0]  IR4;      // The Instruction Register from pipeline stage 3.
   input [31:0]  PC4;      // The Instruction Register from pipeline stage 3.
   input [31:0]  Z4;       // The ALU result from pipeline stage 3.
   input [31:0]  MD4;      // Data to be stored from pipeline stage 3.
   input [31:0]  D_OUT2;   // Memory contents at ADDR_IN2.
   input 	 RW4;      // BWAT: The decoded Register Write bit from pipeline stage 3.
   input 	 HALT4;    // BWAT: The decoded halt bit from pipeline stage 2.
   output [31:0] IR5;      // The Instruction Register for pipeline stage 5.
   output [31:0] Z5;       // The result for pipeline stage 5.
   output [31:0] ADDR_IN2; // The address for memory reads/writes.
   output [31:0] D_IN2;    // The data to write to ADDR_IN2.
   output 	 WR;       // If this instruction is a st or stx, then WR == 1 else WR == 0.
   output 	 BR4;      // If BR4 == 1 then the instruction at pipeline stage 4 is a branch.
   output 	 JMP4;     // If JMP4 == 1 then the instruction at pipeline stage 4 is a jump.
   output 	 RW5;      // BWAT: The decoded Register Write bit for pipeline stage 5.
   output 	 HALT5;    // BWAT: The decoded halt bit for pipeline stage 5.

   // defining internal wires   
   wire [1:0] 	 mp5;
   wire [31:0] 	 z5in;
   wire [31:0] 	 tmp_din2;
   
   d_reg #(1,8) rw5(CLK, RW5, RW4, clr);       // BWAT: Straight forward pass through.
   d_reg #(1,8) halt5(CLK, HALT5, HALT4, clr); // BWAT: Straight forward pass through.
   
   mux2 #(32,6) seldin2(c11, tmp_din2, MD4, Z5);
   mux4 #(32,6) mymux5(mp5[0], mp5[1], z5in, D_OUT2, Z4, PC4, 1);
   d_reg #(32,8) z5reg(CLK, Z5, z5in, clr);
   
   d_reg #(32,8) ir5reg(CLK, IR5, IR4, clr);  // Straight forward pass through.
   
   opdec dec5(IR4[31:27], nop, add, sub, or2, and2, not2, xor2, cmp, br, jmp, ld, ldi, ldx, st, stx, hlt, alu, ldst);
   
   or(WR, st, stx);
   or(mp5[0], alu, ldi, br);
   assign mp5[1] = jmp;
   assign ADDR_IN2 = Z4;
   assign D_IN2 = tmp_din2;
   assign BR4 = br;
   assign JMP4 = jmp;
endmodule // stage4
